  Member
!-----------------------------------------------------------------------------!
! J&S BitList 2 Class Build 1.9901
! Class Source
! Copyright (c) 2000 by J&S Software Co. (jeff@jssoftware.com)
!-----------------------------------------------------------------------------!
! The in-column tagging method as well as the icons have been
! borrowed (with permission) from Mike Hanson's SuperTagging templates.
!-----------------------------------------------------------------------------!
!The contents of this file are subject to the CoveComm Inc. Developers Open 
!Source Public License Version 1.0 (the "License"); you may not use this 
!product except in compliance with the License. You may obtain a copy of 
!the License at http://www.clarionmag.com/common/dosl.html. Software 
!distributed under the License is distributed on an "AS IS" basis, WITHOUT 
!WARRANTY OF ANY KIND, either express or implied. See the License for the 
!specific language governing rights and limitations under the License. 
!The Original Code is Copyright (c) by J&S Software Co., released
!June 28, 1999. The Initial Developer of the Original Code is Jeff Slarve. 
!-----------------------------------------------------------------------------!
! Synopsis
!
! A quick and easy interface to the "bits" available within integers.
! Allows the developer to quickly label each bit and present it to the user
! in the form of a psuedo "check box" instead of declaring a BYTE variable
! and populating a CHECK control for each switch.
!
! The number of records displayed in the listbox are dictated by two things:
!    1. The StartingBit
!    2. The number of items passed in the PARMS parameter of the INIT() method.
!
!-----------------------------------------------------------------------------!


  include('bitlist2.inc')

     map
     end

!=============================================================================================================================
BL2Class.AddItem  Procedure(*? BitMap,String Parms,Byte StartBit=0)
Ndx1      Short,Auto                                                 ! A "placeholder"
Ndx2      Short,Auto                                                 ! A "placeholder"
B         Byte,Auto                                                  ! A Counter
SaveParms &String                                                    ! Temporary String
BitCount  Byte,Auto                                                  ! The number of bits contained in the Bitmap (Usually 1,2, or 4)
TempAny   ANY                                                        ! Making a snapshot of the bitmap

  Code

  TempAny = BitMap                                                   ! Make a copy of the bitmap so that its limits can be tested
  Clear(BitMap,1)                                                    ! Attempt to figure out number of possible bits in this variable by clearing it's value to the highest possible value
  Case BitMap 
  of 255                                                             ! It's a BYTE
    BitCount = 8
  of 32767 OrOf 65535                                                ! SHORT or USHORT
    BitCount = 16 
  else
    BitCount = 32                                                    ! Assume 32 bits
  end
  BitMap = TempAny                                                   ! Restore the original value to the Bitmap
  If NOT Self.Parms &= NULL                                          ! If the Parms string exists
    SaveParms &= NEW(String(Len(Clip(Self.Parms))))                  ! Make space to save the existing Parms
    SaveParms = Self.Parms                                           ! Then copy it
    Dispose(Self.Parms)                                              ! Dispose of the existing Self.Parms so it may be re-created to the new size
    Self.Parms &= NEW(String(Len(Clip(SaveParms)) + Len(Clip(Parms)) + 1)) !Create 
    Self.Parms = Clip(SaveParms) & '|' & Parms                       ! Copy
    Dispose(SaveParms)                                               ! Dispose of temporary string
  else                                                               ! It doesn't exist
    Self.Parms &= NEW(String(Len(Clip(Parms))))                      ! So create a new one
    Self.Parms = Parms                                               ! Then copy it
  end
  If NOT Inrange(StartBit,0,BitCount-1)                              ! Make sure that the startbit is within the legal range
    Message('Bitlist Class: Startbit not within range','Out of Range',ICON:Exclamation)
    StartBit=0                                                       ! If not, then set it to zero
  end
  Ndx1=1                                                             !Initialize our placeholder
  Loop B = StartBit to BitCount - 1                                  !Loop through the parms, and extract the data between the pipe separators
    Clear(Self.ParmQ)
    Self.ParmQ.Bit = B
    Ndx2=Instring('|',Parms,1,Ndx1)                                  !Looking for occurances of the Pipe "|" seperator
    If Ndx2                                                          !If we find it
      Self.ParmQ.Desc=Sub(Parms,Ndx1,Ndx2-Ndx1)                      !then we can grab the data
      Do AddRecord
      Ndx1=Ndx2+1                                                    !Increment the placeholder
    else                                                             !If we don't fined any more Pip "|" seperators
      Self.ParmQ.Desc=Sub(Parms,Ndx1,Len(Clip(Parms))-Ndx1+1)        !we can save the last of our data,
      Do AddRecord
      break                                                          !and break out of our loop
    end
  end
  Self.ReadBits                                                      !Now that we have filled the queue, we can read the corresponding bits in Self.ParmQ.BitMap
  
AddRecord Routine  

  Self.ParmQ.SortDesc=Upper(Self.ParmQ.Desc)                         !
  Self.ParmQ.BitMap        &= BitMap                                 !Assign reference variable to the passed Long
  Self.ParmQ.SaveBitmap     = Self.ParmQ.BitMap                      !Save a copy of the passed Long (just in case)
  Add(Self.ParmQ)                                                    !and add it to the queue.

!=============================================================================================================================
BL2Class.AddMutEx Procedure(Long Line1,Long Line2,Byte Type=BLMutex:Exclusive,Byte Mutual=True)
T Byte,Auto

  Code
  
  Self.FetchMutexRow(Line1)                                          !Grabs the record. Inserts it if it doesn't exist yet.
  Self.FetchRowQRow(Line2)                                           !Grabs the record. Inserts it if it doesn't exist yet.
  T = Type                                                           !Save a copy of the TYPE parameter
  If BAND(T,BLMutex:ParentOffReq)                                    !For the "Parent", we don't want the "required" flag set
    T = BXOR(T,BLMutex:ParentOffReq)                                 !so we shut it off
  end
  If BAND(T,BLMutex:ParentOnReq)                                     !For the "Parent", we don't want the "required" flag set
    T = BXOR(T,BLMutex:ParentOnReq)                                  !so we shut it off   
  end
  Self.MutexQ.RowQ.Type = BOR(Self.MutexQ.RowQ.Type,T)               !Turn on the necessary bits
  Put(Self.MutexQ.RowQ)                                              !Then write to the queue

  If Mutual OR BAND(Type,BLMutex:Exclusive)                          !Repeats the above process for Line2 if necessary
    Self.FetchMutexRow(Line2)
    Self.FetchRowQRow(Line1)
    Self.MutexQ.RowQ.Type = BOR(Self.MutexQ.RowQ.Type,Type)
    Put(Self.MutexQ.RowQ)
  end  

!=============================================================================================================================
BL2Class.CheckBit Procedure(Long Line)
!
!  Test to see if a bit is on or off
!

  Code

  Get(Self.ParmQ,Line)                                               !Get the queue record
  If ErrorCode() then Return 0.                                      !
  Return Choose(BAND(Self.ParmQ.BitMap,BShift(1,Self.ParmQ.Bit))<>0,1,0) !Return TRUE for ON, FALSE for OFF

!=============================================================================================================================
BL2Class.CheckBitStr Procedure(String Desc,Byte Verbose=True)
!
!  Test to see if a bit is on or off (Looking up by description)
!

  Code

  Self.ParmQ.SortDesc = Upper(Desc)
  Get(Self.ParmQ,Self.ParmQ.SortDesc)
  If ErrorCode()
    If Verbose
      Message('The Label "' & Clip(Desc) & '" was not found in the list!|'&|
               'Valid labels are:|' & Self.Parms,'Error', ICON:Exclamation)
    end
    Return(ErrorCode())
  else
    Return(Self.CheckBit(Pointer(Self.ParmQ)))
  end

!=============================================================================================================================
BL2Class.CheckMutEx Procedure
Ndx Long,Auto
P   Long,Auto
On  Byte,Auto
F   Byte,Auto !Exclusive found

  Code

  F = False
  P = Pointer(Self.ParmQ)
  Self.MutexQ.Row = P
  Get(Self.MutexQ,Self.MutexQ.Row)
  If ErrorCode() then Return.
  On = Self.CheckBit(P)
  Loop Ndx = 1 to Records(Self.MutExQ.RowQ)
    Get(Self.MutExQ.RowQ,Ndx)
    If NOT ON
      If NOT BAND(Self.MutexQ.RowQ.Type,BLMutex:AllowClear) AND BAND(Self.MutexQ.RowQ.Type,BLMutex:Exclusive)
        F = True !A mutex was found that disallows clearing of the exclusive bit if it was already on in the first place
      end
    end
    If Self.CheckBit(Self.MutexQ.RowQ.Row)
      Case BAND(Self.MutexQ.RowQ.Type,BLMutex:Exclusive+BLMutex:Off)
      of BLMutex:Exclusive
        Self.FlipBit(Self.MutexQ.RowQ.Row,False)
      of BLMutex:Off
        If NOT On
          Self.FlipBit(Self.MutexQ.RowQ.Row,False)
        end
      else
        If NOT ON
          If BAND(Self.MutexQ.RowQ.Type,BLMutex:On+BLMutex:ParentOnReq)=BLMutex:On+BLMutex:ParentOnReq
            F=True
          end
        end 
      end 
    else
      Case BAND(Self.MutexQ.RowQ.Type,BLMutex:On)
      of BLMutex:On
        If ON
          Self.FlipBit(Self.MutexQ.RowQ.Row,False)
        end
      else  
        If ON
          If BAND(Self.MutexQ.RowQ.Type,BLMutex:Off+BLMutex:ParentOffReq)=BLMutex:Off+BLMutex:ParentOffReq
            F=True
          end
        end 
      end 
    end
  end
  If F
    Self.FlipBit(P,False)
  end
  
!=============================================================================================================================
BL2Class.Construct Procedure

  Code

  Self.ParmQ  &= NEW ParmQ2                                                   !Create a new instance of our queue reference
  Free(Self.ParmQ)                                                            !Make sure that there are no records in it
  Clear(Self.ParmQ)                                                           !Clear the queue record
  Self.MutexQ &= NEW MutexQ                                                   !Create a new instance of the mutex queue
  Free(Self.MutexQ)
  Clear(Self.MutexQ)

!=============================================================================================================================
BL2Class.Destruct Procedure

  Code

  Loop 
    Get(Self.ParmQ,1)
    If ErrorCode() then break.
    Self.ParmQ.BitMap &= NULL
    Delete(Self.ParmQ)
  end
  If NOT Self.ParmQ &= NULL
    Dispose(Self.ParmQ)                                                       !Dispose of the queue
  end  
  If NOT Self.Parms &= NULL
    Dispose(Self.Parms)
  end  
  If NOT Self.MutexQ &= NULL
    Loop
      Get(Self.MutexQ,1)
      If ErrorCode() then break.
      If NOT Self.MutexQ.RowQ &= NULL
        Dispose(Self.MutexQ.RowQ)
      end  
      Delete(Self.MutexQ)
    end
    Dispose(Self.MutexQ) 
  end

!=============================================================================================================================
BL2Class.FetchMutexRow Procedure(Long Line)
!
! Tries to do a GET() on the row. If unsucessful, it adds one.
!
  Code
  
  Self.MutexQ.Row   = Line
  Get(Self.MutexQ,Self.MutexQ.Row)
  If ErrorCode()
    Clear(Self.MutexQ)  
    Self.MutexQ.Row   = Line
    Self.MutexQ.RowQ &= NEW MutexRowQ
    Free(Self.MutexQ.RowQ)
    Clear(Self.MutexQ.RowQ)
    Add(Self.MutexQ,Self.MutexQ.Row)
  end

!=============================================================================================================================
BL2Class.FetchRowQRow Procedure(Long Line)
!
! Tries to do a GET() on the row. If unsucessful, it adds one.
!
  Code
  
  Self.MutexQ.RowQ.Row   = Line
  Get(Self.MutexQ.RowQ,Self.MutexQ.RowQ.Row)
  If ErrorCode()
    Clear(Self.MutexQ.RowQ)
    Self.MutexQ.RowQ.Row   = Line
    Add(Self.MutexQ.RowQ,Self.MutexQ.RowQ.Row)
  end

!=============================================================================================================================
BL2Class.FlipAll Procedure
!
! Flip all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to SELF.Records()                                     !Loop through the bits in the queue
    Self.FlipBit(Ndx)                                                !flip it. flip it good.
  end

!=============================================================================================================================
BL2Class.FlipBit Procedure(Long Line,Byte CheckMutex=True)
!
!  Flips a bit on if off, or off if on.
!

  Code

  Get(Self.ParmQ,Line)                                               !Get the queue record
  If ~ErrorCode()
    Self.ParmQ.BitMap=BXOR(Self.ParmQ.BitMap,BShift(1,Self.ParmQ.Bit)) !Flip the bit
  end
  If CheckMutex
    Self.CheckMutex                                                  ! Check for any mutexes and act accordingly
  end
  Self.ReadBits                                                      ! Refresh the queue

!=============================================================================================================================
BL2Class.HasMutex Procedure
Ndx Long

  Code

  Self.MutexQ.Row = Pointer(Self.ParmQ)                              !Prime the key
  Get(Self.MutexQ,Self.MutexQ.Row)                                   !Fetch the record
  If ErrorCode()                                                     !If there is a problem
    Return False                                                     !Then there certainly isn't a mutex
  end
  Loop Ndx = 1 to Records(Self.MutexQ.RowQ)                          !Examine all of the mutex records for that row
    Get(Self.MutexQ.RowQ,Ndx)
    If BAND(Self.MutexQ.RowQ.Type, BLMutex:Exclusive)                !If there is an exclusive mutex available 
      Return True                                                    !then our job is done
    end
  end
  Return False                                                       !Didn't find any
!=============================================================================================================================
BL2Class.Init Procedure(Short FEQ,Byte ReadOnly_=False,Byte AllowCopy=True)
!
! Initialize the class
!
!

  Code

  Assert(FEQ{PROP:Type}=Create:List)                                 !The FEQ must be a listbox
  Self.BlankBit             = '-'                                    !The Separator Between Active Bits in Self.BitString
  Self.FEQ                  = FEQ                                    !Assign the listbox field equate value to our internal FEQ
  Self.SetReadOnly(ReadOnly_)
  Self.AllowCopy            = AllowCopy
  Self.FEQ{PROP:Alrt,1}     = 0001H                                  !MouseLeft=1
  Self.FEQ{PROP:Alrt,2}     = 0002H                                  !MouseRight=2
  Self.FEQ{PROP:Alrt,3}     = 0005H                                  !MouseLeft2=5
  Self.FEQ{PROP:Alrt,4}     = 0020H                                  !SpaceBar = 0020h
  Self.FEQ{PROP:Alrt,5}     = 0243H                                  !Control-C
  Self.FEQ{PROP:NoBar}      = True
  Self.FEQ{PROP:Format}     = '11C|I@s1@500L(2)@s60@'                !'11C|J@s1@20L(2)@s60@'
  Self.FEQ{PROP:From}       = Self.ParmQ                             !Assign the queue to our listbox
  Self.FEQ{PROP:IconList,1} = '~BoxCheck.ico'                        !Assign the icons to the listbox
  Self.FEQ{PROP:IconList,2} = '~BoxEmpty.ico'
  Self.FEQ{PROP:IconList,3} = '~RadCheck.ico'
  Self.FEQ{PROP:IconList,4} = '~RadEmpty.ico'

!=============================================================================================================================
BL2Class.Init Procedure(Short FEQ,Byte ReadOnly_=False,Byte AllowCopy=True,WindowManager WM)
!
!  If the WindowManager class is passed, it gives the bitlist the opportunity to refresh the screen when the values change.
!
!

  Code

  Self.WM &= WM
  Self.Init(FEQ,ReadOnly_,AllowCopy)

!=============================================================================================================================
BL2Class.Kill Procedure

  Code

  If ~Self.WM &= NULL                                                !
    If Self.WM.Opened                                                !
      Self.FEQ{PROP:From} = ''                                       !
    end                                                              !
  end

!=============================================================================================================================
BL2Class.MarkAll Procedure
!
!  Mark all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to SELF.Records()                                     !Loop through the bits in the queue
    If ~Self.CheckBit(Ndx)                                           !If it isn't "ON"
      Self.FlipBit(Ndx)                                              !flip it to on
    end
  end

!=============================================================================================================================
BL2Class.ReadBits Procedure
!
!  Loops through the designated bits of the bitmap and sets the icon variable in the queue to "Checked" or "UnChecked"
!
Ndx       Byte

  Code

  Loop Ndx = 1 to SELF.Records()                                     !Looping through our queue
    Get(Self.ParmQ,Ndx)                                              !Getting the record
    If Self.HasMutex()
      Self.ParmQ.Icon=Choose(BAND(Self.ParmQ.BitMap,BShift(1,Self.ParmQ.Bit))<>0,3,4,3)!Checking for "on" or "off" and setting the icon variable in the queue
    else
      Self.ParmQ.Icon=Choose(BAND(Self.ParmQ.BitMap,BShift(1,Self.ParmQ.Bit))<>0,1,2,1)!Checking for "on" or "off" and setting the icon variable in the queue
    end
    Self.BitString[Ndx]=Choose(Self.ParmQ.Icon,Self.ParmQ.Desc,Self.BlankBit,Self.ParmQ.Desc,Self.BlankBit,Self.ParmQ.Desc)!Checking for "on" or "off" and setting the icon variable in the queue
    Put(Self.ParmQ)                                                  !writing to the queue
  end

!=============================================================================================================================
BL2Class.Records   Procedure

  Code

  Return Records(Self.ParmQ)
!=============================================================================================================================
BL2Class.SetClipBoard   Procedure
!
!  Copies the currently selected label to the clipboard
!

  Code

  Get(Self.ParmQ,Choice(Self.FEQ))
  SetClipBoard(Self.ParmQ.Desc)

!=============================================================================================================================
BL2Class.SetReadOnly   Procedure(Byte ReadOnly_=1)
Ndx Byte

  Code

  Self.ReadOnly_ = ReadOnly_

!=============================================================================================================================
BL2Class.TakeEvent  Procedure
!
!  Handle accept loop events for the listbox
!
!  Some of this code could probably be moved to seperate methods and cleaned up.
!

  Code

  If (Field()<>Self.FEQ) then Return 0.                              !Make sure we are dealing with the right control and aren't in ReadOnly Mode
  Case Event()
  of Event:PreAlertKey                                               !Need to bypass Event:PreAlertKey
    Return 1                                                         !so we return 1. Otherwise, our keystrokes would get "eaten"
  of Event:AlertKey                                                  !  
    Self.FEQ{PROP:NoBar}=0                                           !So the bar will show when the popup menu displays
    If ~Self.ReadOnly_                                               !
      Case  KeyCode()                                                !If the keycode() is..
      of 1 OROF 5                                                    !Left mouse button
        If Self.FEQ{PROPLIST:MouseDownRow} AND Self.FEQ{PROPLIST:MouseDownZone} <> LISTZONE:Right | !Is in our "checkbox" area..
                                           AND Self.FEQ{PROPLIST:MouseDownField} = 1 OR KeyCode() = 5
          Self.FlipBit(Self.FEQ{Proplist:MouseDownRow})              !Then we will flip the bit
        end
      of 2                                                           !Right mouse button
        Execute Popup('Check All|Uncheck All|Flip All'&|
                      Choose(~Self.AllowCopy,'','|-|Copy to Clipboard'))      !We will display a popup menu
          Self.MarkAll                                               !Turn all bits that are in the listbox on.
          Self.UnMarkAll                                             !Turn all bits that are in the listbox off.
          Self.FlipAll                                               !Flip all bits in the listbox
          Self.SetClipBoard()                                        !Copy current selection's label to the clipboard
        end
        Select( Self.FEQ )                                           !Select the listbox
      of 0020h                                                       !SpaceBar
        Self.FlipBit(Choice(Self.FEQ))                               !Flip the bit
        Self.FEQ{PROP:Selected} = Choose(Self.FEQ{PROP:Selected} < SELF.Records(),|!If we aren't on the last row then
                                 Self.FEQ{PROP:Selected}+1,Self.FEQ{PROP:Selected})!we will select the next record down
      of 0243H 
        If Self.AllowCopy
          Self.SetClipBoard()
        end
      end
      If NOT Self.WM &= NULL
        Self.WM.Reset
      end
    else                                                             !If it is ReadOnly, we can still let them copy the selection
      If Self.AllowCopy 
        Case KeyCode() 
        of 2                                                         !MoustLeft2
          If Popup('Copy to ClipBoard')
            Self.SetClipBoard
          end
        of 0243H                                                     !Control C
          Self.SetClipBoard
        end
      end
    end
    Self.FEQ{PROP:NoBar}=1                                           !Reset the No Bar
  end
  Return 0                                                           !Return zero

!=============================================================================================================================
BL2Class.UnMarkAll Procedure
!
!  UnMark all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to SELF.Records()                                     !Loop through the bits in the queue
    If Self.CheckBit(Ndx)                                            !If it isn't "Off"
      Self.FlipBit(Ndx)                                              !flip it to off
    end
  end


