  Member
_ABCDllMode_  EQUATE(0)
_ABCLinkMode_ EQUATE(1)
!-----------------------------------------------------------------------------!
! J&S BitList Class .9912
! Class Source
! Copyright (c) 1999 by J&S Software Co. (jeff@jssoftware.com)
!-----------------------------------------------------------------------------!
! The in-column tagging method as well as the icons have been
! borrowed (with permission) from Mike Hanson's SuperTagging templates.
!-----------------------------------------------------------------------------!
!The contents of this file are subject to the CoveComm Inc. Developers Open 
!Source Public License Version 1.0 (the "License"); you may not use this 
!product except in compliance with the License. You may obtain a copy of 
!the License at http://www.clarionmag.com/common/dospl.html. Software
!distributed under the License is distributed on an "AS IS" basis, WITHOUT 
!WARRANTY OF ANY KIND, either express or implied. See the License for the 
!specific language governing rights and limitations under the License. 
!The Original Code is Copyright (c) by J&S Software Co., released
!June 28, 1999. The Initial Developer of the Original Code is Jeff Slarve. 
!-----------------------------------------------------------------------------!
! Synopsis
!
! A quick and easy interface to the 32-bits available within a LONG integer.
! Allows the developer to quickly label each bit and present it to the user
! in the form of a psuedo "check box" instead of declaring a BYTE variable
! and populating a CHECK control for each switch.
!
! The number of records displayed in the listbox are dictated by two things:
!    1. The StartingBit
!    2. The number of items passed in the PARMS parameter of the INIT() method.
!
!-----------------------------------------------------------------------------!


  include('bitlist.inc')

     map
     end

!=============================================================================================================================
BLClass.CheckBit Procedure(Long Line)
!
!  Test to see if a bit is on or off
!

  Code

  Get(Self.ParmQ,Line)                                                        !Get the queue record
  If ErrorCode() then Return 0.                                               !
  Return Choose(BAND(Self.BitMap,BShift(1,Self.ParmQ.Bit))<>0,1,0)            !Return TRUE for ON, FALSE for OFF

!=============================================================================================================================
BLClass.CheckBitStr Procedure(String Desc,Byte Verbose=True)
!
!  Test to see if a bit is on or off (Looking up by description)
!

  Code

  Self.ParmQ.SortDesc = Upper(Desc)
  Get(Self.ParmQ,Self.ParmQ.SortDesc)
  If ErrorCode()
    If Verbose
      Message('The Label "' & Clip(Desc) & '" was not found in the list!|'&|
               'Valid labels are:|' & Self.Parms,'Error', ICON:Exclamation)
    end
    Return(ErrorCode())
  else
    Return(Self.CheckBit(Pointer(Self.ParmQ)))
  end

!=============================================================================================================================
BLClass.FlipAll Procedure
!
! Flip all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to Self.Records                                              !Loop through the bits in the queue
    Self.FlipBit(Ndx)                                                       !flip it
  end

!=============================================================================================================================
BLClass.FlipBit Procedure(Long Line)
!
!  Flips a bit on if off, or off if on.
!

  Code

  Get(Self.ParmQ,Line)                                                        !Get the queue record
  If ~ErrorCode()
    Self.BitMap=BXOR(Self.BitMap,BShift(1,Self.ParmQ.Bit))                    !Flip the bit
  end
  Self.ReadBits                                                               !Refresh the queue

!=============================================================================================================================
BLClass.Init Procedure(*Long BitMap,String Parms,Short FEQ,Byte StartBit=0,Byte ReadOnly_=False,Byte AllowCopy=True)
!
! Initialize the class
!
! Usage:  Object.Init(YourLongVariable,'A String|Containing|Pipe symbol|separated|labels',?AListBox [,TheStartingBit] [, ReadOnly] [, AllowCopyToClipBoard])
!
!

  Code

  Assert(FEQ{PROP:Type}=Create:List)                                          !The FEQ must be a listbox
  Self.BitMap              &= BitMap                                          !Assign reference variable to the passed Long
  Self.SaveBitmap           = Self.BitMap                                     !Save a copy of the passed Long (just in case)
  Self.BlankBit             = '-'                                             !The Separator Between Active Bits in Self.BitString
  Self.Parms                = Parms                                           !Copy the passed parameters to an internal string to be parsed later
  Self.ParmQ               &= NEW ParmQ                                       !Create a new instance of our queue reference
  Free(Self.ParmQ)                                                            !Make sure that there are no records in it
  Clear(Self.ParmQ)                                                           !Clear the queue record
  Self.FEQ                  = FEQ                                             !Assign the listbox field equate value to our internal FEQ
  Self.SetReadOnly(ReadOnly_)
  Self.AllowCopy            = AllowCopy
  Self.FEQ{PROP:Alrt,1}     = 0001H                                           !MouseLeft=1
  Self.FEQ{PROP:Alrt,2}     = 0002H                                           !MouseRight=2
  Self.FEQ{PROP:Alrt,3}     = 0005H                                           !MouseLeft2=5
  Self.FEQ{PROP:Alrt,4}     = 0020H                                           !SpaceBar = 0020h
  Self.FEQ{PROP:Alrt,5}     = 0243H                                           !Control-C
  Self.FEQ{PROP:NoBar}      = True
  Self.FEQ{PROP:Format}     = '11C|I@s1@500L(2)@s60@'                         !'11C|J@s1@20L(2)@s60@'
  Self.FEQ{PROP:From}       = Self.ParmQ                                      !Assign the queue to our listbox
  Self.FEQ{PROP:IconList,1} = '~BoxCheck.ico'                                 !Assign the icons to the listbox
  Self.FEQ{PROP:IconList,2} = '~BoxEmpty.ico'
  Self.StartBit             = Choose(Inrange(StartBit,0,31), StartBit, 0)     !If the StartBit was passed, we will save that value
  Self.ParseParms                                                             !Parse Self.Parms and load the queue
  Self.ReadBits                                                               !Now that we have filled the queue, we can read the corresponding bits in Self.BitMap

!=============================================================================================================================
BLClass.Init Procedure(*Long BitMap,String Parms,Short FEQ,Byte StartBit=0,Byte ReadOnly_=False,Byte AllowCopy=True,WindowManager WM)
!
!  If the WindowManager class is passed, it gives the bitlist the opportunity to refresh the screen when the values change.
!
!

  Code

  Self.WM &= WM
  Self.Init(BitMap,Parms,FEQ,StartBit,ReadOnly_,AllowCopy)

!=============================================================================================================================
BLClass.Kill Procedure

  Code

  If ~Self.WM &= NULL                                                         !
    If Self.WM.Opened                                                         !
      Self.FEQ{PROP:From} = ''                                                !Version .9912 to fix sporadic GPF
    end                                                                       !
  end
  Free(Self.ParmQ)                                                            !Empty the queue records. (This is implicitly done by the dispose(), but old habits are tough to break)
  Dispose(Self.ParmQ)                                                         !then dispose of the queue

!=============================================================================================================================
BLClass.MarkAll Procedure
!
!  Mark all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to Self.Records                                                !Loop through the bits in the queue
    If ~Self.CheckBit(Ndx)                                                    !If it isn't "ON"
      Self.FlipBit(Ndx)                                                       !flip it to on
    end
  end

!=============================================================================================================================
BLClass.ParseParms Procedure
!
! Parses the parms and fills the queue with their data. (Called one time by the INIT() method)
!
Ndx1      Short                                                               ! A "placeholder"
Ndx2      Short                                                               ! A "placeholder"

  Code

  Ndx1=1                                                                      !Initialize our placeholder
  Loop Self.ParmQ.Bit = Self.StartBit to 31                                   !Loop through the parms, and extract the data between the pipe separators
    Ndx2=Instring('|',Self.Parms,1,Ndx1)                                      !Looking for occurances of the Pipe "|" seperator
    If Ndx2                                                                   !If we find it
      Self.ParmQ.Desc=Sub(Self.Parms,Ndx1,Ndx2-Ndx1)                          !then we can grab the data
      Self.ParmQ.SortDesc=Upper(Self.ParmQ.Desc)                              !
      Add(Self.ParmQ)                                                         !and add it to the queue.
      Ndx1=Ndx2+1                                                             !Increment the placeholder
    else                                                                      !If we don't fined any more Pip "|" seperators
      Self.ParmQ.Desc=Sub(Self.Parms,Ndx1,Len(Clip(Self.Parms))-Ndx1+1)       !we can save the last of our data,
      Self.ParmQ.SortDesc=Upper(Self.ParmQ.Desc)                              !
      Add(Self.ParmQ)                                                         !add it to the queue,
      break                                                                   !and break out of our loop
    end
  end
  Self.Records=Records(Self.ParmQ)                                            !Since the number of records in the queue is to remain fixed, we can save this value so Records() doesn't have to be called repeatedly

!=============================================================================================================================
BLClass.ReadBits Procedure
!
!  Loops through the designated bits of the bitmap and sets the icon variable in the queue to "Checked" or "UnChecked"
!
Ndx       Byte

  Code

  Loop Ndx = 1 to Self.Records                                                 !Looping through our queue
    Get(Self.ParmQ,Ndx)                                                        !Getting the record
    Self.ParmQ.Icon=Choose(BAND(Self.BitMap,BShift(1,Self.ParmQ.Bit))<>0,1,2,1)!Checking for "on" or "off" and setting the icon variable in the queue
    Self.BitString[Ndx]=Choose(Self.ParmQ.Icon,Self.ParmQ.Desc,Self.BlankBit,Self.ParmQ.Desc)!Checking for "on" or "off" and setting the icon variable in the queue
    Put(Self.ParmQ)                                                            !writing to the queue
  end

!=============================================================================================================================
BLClass.SetClipBoard   Procedure
!
!  Copies the currently selected label to the clipboard
!

  Code

  Get(Self.ParmQ,Choice(Self.FEQ))
  SetClipBoard(Self.ParmQ.Desc)

!=============================================================================================================================
BLClass.SetReadOnly   Procedure(Byte ReadOnly_=1)
Ndx Byte

  Code

  Self.ReadOnly_ = ReadOnly_

!=============================================================================================================================
BLClass.TakeEvent  Procedure
!
!  Handle accept loop events for the listbox
!
!  Some of this code could probably be moved to seperate methods and cleaned up.
!

  Code

  If (Field()<>Self.FEQ) then Return 0.                        !Make sure we are dealing with the right control and aren't in ReadOnly Mode
  Case Event()
  of Event:PreAlertKey                                                        !Need to bypass Event:PreAlertKey
    Return 1                                                                  !so we return 1. Otherwise, our keystrokes would get "eaten"
  of Event:AlertKey
    Self.FEQ{PROP:NoBar}=0                                                    !So the bar will show when the popup menu displays
    If ~Self.ReadOnly_                                                        !
      Case  KeyCode()                                                         !If the keycode() is..
      of 1 OROF 5                                                             !Left mouse button
        If Self.FEQ{PROPLIST:MouseDownRow} AND Self.FEQ{PROPLIST:MouseDownZone} <> LISTZONE:Right | !Is in our "checkbox" area..
                                           AND Self.FEQ{PROPLIST:MouseDownField} = 1 OR KeyCode() = 5
          Self.FlipBit(Self.FEQ{Proplist:MouseDownRow})                       !Then we will flip the bit
        end
      of 2                                                                    !Right mouse button
        Execute Popup('Check All|Uncheck All|Flip All'&|
                      Choose(~Self.AllowCopy,'','|-|Copy to Clipboard'))      !We will display a popup menu
          Self.MarkAll                                                        !Turn all bits that are in the listbox on.
          Self.UnMarkAll                                                      !Turn all bits that are in the listbox off.
          Self.FlipAll                                                        !Flip all bits in the listbox
          Self.SetClipBoard()                                                 !Copy current selection's label to the clipboard
        end
        Select( Self.FEQ )                                                    !Select the listbox
      of 0020h                                                                !SpaceBar
        Self.FlipBit(Choice(Self.FEQ))                                        !Flip the bit
        Self.FEQ{PROP:Selected} = Choose(Self.FEQ{PROP:Selected} < Self.Records,|!If we aren't on the last row then
                                 Self.FEQ{PROP:Selected}+1,Self.FEQ{PROP:Selected})!we will select the next record down
      of 0243H 
        If Self.AllowCopy
          Self.SetClipBoard()
        end
      end
      If NOT Self.WM &= NULL
        Self.WM.Reset
      end
    else                                                                      !If it is ReadOnly, we can still let them copy the selection
      If Self.AllowCopy 
        Case KeyCode() 
        of 2                                                                  !MoustLeft2
          If Popup('Copy to ClipBoard')
            Self.SetClipBoard
          end
        of 0243H                                                              !Control C
          Self.SetClipBoard
        end
      end
    end
    Self.FEQ{PROP:NoBar}=1                                                    !Reset the No Bar
  end
  Return 0                                                                    !Return zero

!=============================================================================================================================
BLClass.UnMarkAll Procedure
!
!  UnMark all bits that exist in the queue
!
Ndx       Byte

  Code

  Loop Ndx = 1 to Self.Records                                              !Loop through the bits in the queue
    If Self.CheckBit(Ndx)                                                   !If it isn't "Off"
      Self.FlipBit(Ndx)                                                     !flip it to off
    end
  end

