// example3.cpp : Defines the entry point for the DLL application.
//

#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>
#include <fstream>

//  IOutputFile Declaration  ---
class IOutputFile
{
public:
	virtual void __stdcall appendText(const char *text) = 0;
	virtual void __stdcall appendLine(const char *line) = 0;
};

//  IOutputFile Implementation Declaration  ---
class OutputStream : public IOutputFile
{
public:
	OutputStream(const char * filePath)
	{
		ofs = new std::ofstream(filePath);
	}
	~OutputStream()
	{
		delete(ofs);
	}
	virtual void __stdcall appendText(const char * text)
	{
		*ofs << text; 
	}
	virtual void __stdcall appendLine(const char *line)
	{
		*ofs << line << "\r\n"; 
	}
	
private:
	std::ofstream * ofs;
};

//  NEW and DISPOSE Defintions  ---
extern "C" __declspec(dllexport) IOutputFile * __stdcall newOutputFile(const char * filePath)
{
	return new OutputStream(filePath);
}

extern "C" __declspec(dllexport) void __stdcall disposeOutputFile(IOutputFile * s)
{
	delete((OutputStream *)s);
	return;
}

//  Required for all dlls  ---
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}
