    ! Clarion Magazine String Reference Example 8
    ! 6-15-2003
    !
    ! Demonstrates the use of String and CString References when dealing with a Win32 API call.
    ! Also demonstrate the technique of using GROUPs over the &STRING to force a Size to a &STRING.
    ! Items stored as CString References in a Queue.


    PROGRAM

    MAP
      MODULE('Win32API')
GetEnvironmentStrings   PROCEDURE(),LONG,PASCAL,NAME('GetEnvironmentStringsA')
FreeEnvironmentStrings  PROCEDURE(*STRING TheBuffer),SIGNED,RAW,PASCAL,PROC,NAME('FreeEnvironmentStringsA')
      END !MODULE
    END !MAP

RefGroup      GROUP
MyBuffer        &STRING    ! The memory buffer
              END !GROUP

OverRefGroup    GROUP,OVER(RefGroup)
TheAddress        LONG
TheSize           SIGNED
                END !GROUP

BufferSize      LONG        ! It's size
StringCount     LONG        ! The count of CSTRINGs in MyBuffer

TempCString     &CSTRING    ! A CSTRING ref. Var.
TempAddress     LONG,AUTO    ! A memory address

StorageQ        QUEUE        ! The &CSTRING Storage Queue
TheCString        &CSTRING
                END !QUEUE

DisplayQ        QUEUE        ! The CSTRING Display Queue
TheCString        CSTRING(255)
                END !QUEUE

Counter         LONG        ! A LOOP counter

    ! Define our Display Window.
Window WINDOW('GetEnvironmentStrings Example'),     |
               AT(,,316,122),IMM,SYSTEM,GRAY
       STRING('StorageQ of &CSTRINGs'),AT(6,4),USE(?String1)
       STRING('DisplayQ of Fixed Length CSTRINGs'),  |
               AT(163,3),USE(?String2)
       LIST,AT(3,16,152,82),USE(?List1),VSCROLL,    |
               FORMAT('255L(2)|MS(255)'),FROM(StorageQ)
       LIST,AT(160,16,152,82),USE(?List2),VSCROLL,  |
               FORMAT('1024L(2)|MS(1024)'),FROM(DisplayQ)
       PROMPT('Buffer Size:'),AT(4,107),USE(?Prompt1)
       STRING(@N10),AT(46,107),USE(BufferSize)
       STRING('String Count:'),AT(118,107),USE(?String4)
       STRING(@N_10),AT(168,107),USE(StringCount)
     END

        CODE
        ! Get the Memory Address of the Environment String Buffer
        ! and cast that address as a &STRING. This will be used
        ! in the FreeEnvironmentStrings function later.
        RefGroup.MyBuffer &= ( GetEnvironmentStrings() )
        
        ! Initialize some LOOP variables.
        !    1) Get the Address of the STRING Buffer.
        TempAddress = ADDRESS(RefGroup.MyBuffer)
        !    2) Reference assign (by address) the first CSTRING.
        !        At this point TempCString refers to the first
        !        CSTRING in the MyBuffer, the LEN() will be the
        !        length of that CSTRING due to the ASCII zero
        !        terminating character. Note: SIZE(TempCString)
        !        would be zero if we were testing on it.
        TempCString &= ( TempAddress )

        ! Since &CSTRINGs have a Length irrespective of Size
        ! We can use that to determine when we have found the final
        ! NULL CSTRING at the end of the buffer.
        LOOP WHILE LEN(TempCString) <> 0

        ! Add the LEN() to the BufferSize, taking into account the 
        ! terminating ASCII zero.
          BufferSize = BufferSize + (LEN(TempCString) + 1)

        ! Add to the StringCount
          StringCount += 1

        ! Reference Assign the TempCString to the Storage
        ! Queue's &CSTRING and add it to the QUEUE.
          StorageQ.TheCString &= TempCString
          ADD(StorageQ)

        ! Move the TempAddress Pointer to the end of
        ! the current CSTRING and reassign the TempCString,
        ! once again, by using assign-by-address "casting".
          TempAddress = TempAddress + LEN(TempCString) + 1
          TempCString &= ( TempAddress )
        END !LOOP
    
        ! Add the last NULL byte to the BufferSize
        BufferSize += 1

        ! Force the SIZE component of the Reference variable
        OverRefGroup.TheSize = BufferSize

        ! Display some info about it.
        MESSAGE('The SIZE() of RefGroup.MyBuffer): ' & SIZE(RefGroup.MyBuffer) & '<13,10> and the first three bytes of it is "' & RefGroup.MyBuffer[1 : 3])

        ! Now that we have a QUEUE of &CSTRINGs we can do
        ! anything we need to do. Like create a Display Queue
        ! to show in a ListBox.
        LOOP Counter = 1 TO StringCount
          GET(StorageQ,Counter)
          DisplayQ.TheCString = StorageQ.TheCString
          ADD(DisplayQ)
        END !LOOP
    
        ! Open the Window and Start and ACCEPT loop.
        ! This is a simple display window, so the ACCEPT
        ! loop will automatically BREAK upon an EVENT:CloseWindow
        OPEN(Window)
        ACCEPT
        END !ACCEPT
        
        ! Free the Storage. Notice that, since we did not allocate
        ! any memory for the StorageQ &CSTRINGs we do not have to
        ! do anything special in FREEing the Queue. If we had used
        ! allocated memory or ANY's we would have to LOOP through
        ! the StorageQ to DISPOSE or NULL the variables.
        FREE(DisplayQ)
        FREE(StorageQ)

        ! Free the Environment String buffer now that we are done.
        FreeEnvironmentStrings(RefGroup.MyBuffer)

        RETURN

